"Script for logging encryption/decryptions, login/logout dates, and password changes for the Revenant app."

import os
from getpass import getuser
import datetime as dt

class Log:
    """
    A class to audit the userLog file and log things such as login/logout dates.
    """


    def __init__(self, username) -> None:
        self.username = username
    

    def audit(self, mode=str) -> int:
        """
        Function to audit the UserLog file.

        Parameters:
            s: Removes single file log entries.
            f: Removes folder log entries.
            v: Removes Vault log entries.
            a: Removes all entries.

        Return Codes:
            Code of 0: no errors detected.
            Code of 1: Mode was not specified.
        """
        if mode is None:
            return 1
        #s for singular, f for folders, v for vault, a for all
        if mode == "s":
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'r') as file:
                lines = file.readlines()
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'w') as file:
                for line in lines:
                    if "Singular:" not in line.strip("\n"):
                        file.write(line)
            return 0
        elif mode == "f":
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'r') as file:
                lines = file.readlines()
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'w') as file:
                for line in lines:
                    if "Folder:" not in line.strip("\n"):
                        file.write(line)
            return 0
        elif mode == "v":
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'r') as file:
                lines = file.readlines()
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'w') as file:
                for line in lines:
                    if "Vault" not in line.strip("\n"):
                        file.write(line)
            return 0
        elif mode == "a":
            with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'w') as file:
                file.write("Userlog cleared.\n")
            return 0
        pass
    

    def auto_audit(self) -> int:
        """
        a simple function that runs at runtime to make sure the userlog file does not get too large.
        """
        assumed_username = getuser()
        try:
            size = os.path.getsize("/home/{}/Revenant_source_code/userLog.txt".format(assumed_username))
            if size >= 64000:
                self.audit("a")
                return 1
        except FileNotFoundError:
            pass


    def log(self, mode=str, path=str, encrypted=bool) -> int:
        """
        Logging function for various file operations.

        Parameters:
            mode (string): This is the file operation you are doing.
                Single files = "s"   \n
                Folders = "f"   \n
                Vault operations = "v"   \n
                password change = "c"   \n
            path (string): The file path of the file you have done an operation on.

            encrypted (boolean): Whether or not the operation was an encryption.

        Returns 0 if logging was successful, 1 if mode or path was None, and 2 if userLog does not exist.
        """
        if mode is None or path is None:
            return 1
        time_ = str(dt.datetime.utcnow())
        try:
            if mode == "s":
                with open ("/home/{}/Revenant/userLog.txt".format(self.username), "a") as file:
                    if encrypted is True:
                        string_ = "encrypted"
                    else:
                        string_ = "decrypted"
                    file.write("Singular: File: {} was: {} at: {} UTC\n".format(path, string_, time_))
                pass
            elif mode == "f":
                with open("/home/{}/Revenant/userLog.txt".format(self.username) ,'a') as file:
                    # formatting
                    if encrypted is True:
                        string_ = "encrypted"
                    elif encrypted is False:
                        string_ = "decrypted"
                    # writing
                    file.write("Folder: Folder: {} was: {} at: {} UTC.\n".format(path, string_, time_))
            elif mode == "v":
                with open("/home/{}/Revenant/userLog.txt".format(self.username) ,'a') as file:
                    # formatting
                    if encrypted is True:
                        string_ = "encrypted"
                    elif encrypted is False:
                        string_ = "decrypted"
                    # writing
                    file.write("Vault: Vault was: {} at: {} UTC.\n".format(string_, time_))
            elif mode == "c":
                with open("/home/{}/Revenant/userLog.txt".format(self.username) ,'a') as file:
                    file.write("Password: password was changed at: {} UTC.\n".format(time_))
        except FileNotFoundError:
            file = open("/home/{}/Revenant/userLog.txt".format(self.username), "w+")
            file.write("userLog recreated due to FileNotFoundError.\n")
            file.close()
            return 2


    def log_logout(self) -> int:
        """
        A small function to log the logout date.
        
        Returns 0.
        """
        logout_date = dt.datetime.utcnow()
        with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'a') as file:
            file.write("Logout detected at: " + str(logout_date) + " UTC.\n")
        return 0


    def log_login(self) -> int:
        """
        A small function to log the login date of a user.
    
        Returns 0.
        """
        login_date = dt.datetime.utcnow()
        with open ("/home/{}/Revenant/userLog.txt".format(self.username), 'a') as file:
            file.write("Login detected at: " + str(login_date) + " UTC.\n")
        return 0
